@require(symbol, editing_link)

@(
tags = {}
if symbol.comment:
    tags = symbol.comment.tags

tags.pop('returns', None)
)

<div class="base_symbol_container" data-hotdoc-tags="\
@for tag in tags.values():
@tag.name.lower():@tag.description;\
@end
" id="@symbol.link.id_">
	<ul class="base_symbol_header">
		<li class="base_symbol_name">
		@def header():
		@end
		@header()
		</li>
		@if editing_link:
		@editing_link
		@end
	</ul>
@def content():
@end
@content()
</div>
