@require(toc_sections)
<div>
	<table class="symbols_toc">
		@for toc_section in toc_sections:
			<th colspan=3>@toc_section.name</th>
			@for summary in toc_section.summaries:
			@summary
			@end
		\
		@end
	</table>
</div>
