@require(page, assets_path, scripts, site_navigation)

<!DOCTYPE html>
<html lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<meta http-equiv="X-UA-Compatible" content="IE=edge" />
<meta name="viewport" content="width=device-width, initial-scale=1" />

<title>@page.get_title()</title>

<link rel="stylesheet" href="@assets_path/css/frontend.css" type="text/css"/>
<script src="@assets_path/js/frontend.js"></script>

@for script in scripts:
<script src="@assets_path/js/@script"></script>
@end

</head>

<body data-spy="scroll" data-target=".scrollspy" data-offset="70">
<div data-extension="@str(page.extension_name)" class="page_container" id="page-wrapper">

	<nav class="navbar navbar-default navbar-fixed-top">
		<div class="container-fluid">
			<div class="navbar-header">
				<button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar-wrapper" aria-expanded="false">
					<span class="sr-only">Toggle navigation</span>
					<span class="icon-bar"></span>
					<span class="icon-bar"></span>
					<span class="icon-bar"></span>
				</button>
			</div>
			<div class="collapse navbar-collapse" id="navbar-wrapper">
				<div class="btn-group" id="menu">
				</div>
			</div>
		</div>
	</nav>


	<div class="row">
		<div class="col-md-2" id="sidenav-column">
			<div class="panel panel-collapse" id="sidenav">
				<div id="sidenav-search" role="search">
					<input type="text" placeholder="Fetching index" id="sidenav-lookup-field" disabled="disabled"/>
				</div>
				<div class="sidenav-main-panel-body">
					<div>
						@if site_navigation:
						@site_navigation
						@end
					</div>
				</div>
			</div>
		</div>


		<div class="col-md-8">
			<div id="main">
				@def content():
				@end
				@content()
			</div>
			<div id="search_results">
				<p>The results of the search are</p>
			</div>
		</div>
		<div class="col-md-2">
		</div>
	</div>
</div>
</body>
</html>
