@require(prototype, property, extra)
@extends('base_symbol.html')

@def header():
<h3 class="method">
	<span>The <code>“@property.link.title”</code> property</span>
</h3>
@end
@def content():
@prototype
@property.formatted_doc

@if extra:
\
@for name, ex in extra.iteritems():
<p>
	@name :\
	@ex
</p>
@end
@end
@end
