@require(return_value)
<div class="return_value_details">
	<h4>Returns</h4>
	@return_value.formatted_doc
</div>

@if return_value.extension_contents:
<p>
	@for name, ex in return_value.extension_contents.iteritems():
	@ex
	@end
</p>
@end
