@require(struct, raw_code, members_list)
@extends('base_symbol.html')

@def header():
<h3>
	struct @struct.link.title
</h3>
@end
@def content():
\
@if raw_code:
@raw_code
@end
\
@if struct.formatted_doc:
@struct.formatted_doc
@end
@members_list
@end
