@require(prototype, symbol, return_value, parameters, callable_type, tags, extra)
@extends('base_symbol.html')

@def header():
<h3>
	<span>The <code>“@symbol.link.title”</code> @callable_type</span>
</h3>
@end
@def content():
@prototype
@symbol.formatted_doc
@if parameters:
	<div class="parameter_container">
		<h4>Parameters</h4>
		<table>
			<tbody>
				@for parameter in parameters:
					@parameter
				@end
			</tbody>
		</table>
	</div>
@end

@if return_value:
	@return_value
@end

@if extra:
\
@for name, ex in extra.iteritems():
<p>
	@name :\
	@ex
</p>
@end
@end

@for tag in tags.values():
	<p>
	@tag.name\
	@if tag.description:
		: @tag.description
	@end
	</p>
@end
\
@end
