@require(klass, hierarchy)
@extends('base_symbol.html')

@def header():
<h3>
	@klass.link.title
</h3>
@end

@def content():
\
@if hierarchy:
<div class="hierarchy_container">
	@hierarchy
</div>
@end

@if klass.formatted_doc:
<div class="class_details">
	@klass.formatted_doc
</div>
@end
\
@end
