@require(definition, constant)
@extends('base_symbol.html')

@def header():
<h3 class="constant">
	@constant.link.title
</h3>
@end
@def content():
\
@if definition:
@definition
@end
@constant.formatted_doc
@end
