@require(page, toc, symbols_details)
@extends('base_page.html')

@def content():

@if page.formatted_contents:
<div id="page-description">
@page.formatted_contents
</div>
@end

@toc

@for symbols_detail in symbols_details:
@if symbols_detail.name:
@(html_id = '_'.join(symbols_detail.name.split()))
<h2 class="symbol_section" id="@html_id">@symbols_detail.name</h2>
@end
\
@for i, description in enumerate(symbols_detail.descriptions):
@description
@end
@end
\
@end
