@require(alias, aliased_type)
@extends('base_symbol.html')

@def header():
<h3>
	@alias.link.title
</h3>
@end
@def content():
<pre>
typedef @aliased_type @alias.link.title\
</pre>
@alias.formatted_doc
@end
