@require(return_values)
<div class="multi_return_value_container">
	@if len (return_values) == 1:
		<h4>Returns:</h4>
	@else:
		<h4>Returns a tuple made of</h4>
	@end
	<table>
		<colgroup>
			<col/>
			<col/>
			<col/>
		</colgroup>
		<tbody>
			@for value in return_values:
			<tr>
				<td>
					<p>
					@if hasattr(value, "argname"):
					<i>@value.argname:</i>
					@end
					</p>
				</td>
				<td>
					@value.formatted_doc
				</td>
				<td>
					@value.formatted_link
				</td>
			</tr>
			@end
		</tbody>
	</table>
</div>
