@require(property_type, property_link, extra_contents)
@extends('base_symbol_summary.html')

@def type():
\
    @if property_type:
	<span>@property_type</span>
    @end
@end
@def name():
<span>@property_link</span>
@end
@def extra():
\
    @for name, ex in extra_contents.iteritems():
    @ex
    @end
@end
