@require(page, scripts, assets_path)

<!DOCTYPE html>
<html lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />

<title>@page.get_title()</title>

@for script in scripts:
<script src="@assets_path/js/@script"></script>
@end
</head>

<div class="page_container">
	@def content():
	@end
	@content()
</div>
</html>
