@require(return_value, function_name, is_callable, is_pointer)
@extends('base_symbol_summary.html')

@def type():
\
@if return_value:
	<span>@return_value</span>
@end
\
@end
@def name():
<span>\
@if is_pointer:
<span>(*</span>\
@end
@function_name\
@if is_pointer:
<span>)</span>\
@end

@if is_callable:
<span>()</span>\
@end
</span>
@end
